package com.example.demo;

import java.time.LocalDateTime;
import java.util.List;
import java.util.Objects;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.boot.CommandLineRunner;
import org.springframework.boot.SpringApplication;
import org.springframework.boot.autoconfigure.SpringBootApplication;

import com.example.demo.messages.model.MessageEntity;
import com.example.demo.messages.service.MessageService;
import com.example.demo.orders.model.OrderEntity;
import com.example.demo.orders.service.OrderService;
import com.example.demo.types.model.TypeEntity;
import com.example.demo.types.service.TypeService;
import com.example.demo.users.model.UserEntity;
import com.example.demo.users.model.UserRole;
import com.example.demo.users.service.UserService;

@SpringBootApplication
public class DemoApplication implements CommandLineRunner {
	// Логгер
	private final Logger log = LoggerFactory.getLogger(DemoApplication.class);

	// Бизнес-логика для сущности "Тип" (Тип книг)
	private final TypeService typeService;

	// Бизнес-логика для сущности "Заказ" (Заказ, содержащий книги)
	private final OrderService orderService;

	// Бизнес-логика для сущности "Пользователь"
	private final UserService userService;

	// Бизнес-логика для сущности "Сообщение"
	private final MessageService messageService;

	// Конструктор
	public DemoApplication(
			TypeService typeService,
			OrderService orderService,
			UserService userService,
			MessageService messageService) {
		this.typeService = typeService;
		this.orderService = orderService;
		this.userService = userService;
		this.messageService = messageService;
	}

	// Входная точка программы
	public static void main(String[] args) {
		SpringApplication.run(DemoApplication.class, args);
	}

	@Override
	public void run(String... args) throws Exception {
		if (args.length > 0 && Objects.equals("--populate", args[0])) {
			log.info("Create default types values");
			final var type1 = typeService.create(new TypeEntity("Protection"));
			final var type2 = typeService.create(new TypeEntity("Sharpness"));
			final var type3 = typeService.create(new TypeEntity("Infinity"));

			log.info("Create default users values");
			final var user1 = userService.create(new UserEntity("User1", "password", "mail1@gmail.com"));
			final var user2 = userService.create(new UserEntity("User2", "password", "mail2@gmail.com"));
			final var user3 = userService.create(new UserEntity("User3", "password", "mail3@gmail.com"));
			final var admin = new UserEntity("admin", "admin", "admin@gmail.com");
			admin.setRole(UserRole.ADMIN);
			userService.create(admin);

			log.info("Create default order values");
			final var orders = List.of(
				new OrderEntity(type1, 12.00, 3),
				new OrderEntity(type1, 50.00, 20),
				new OrderEntity(type2, 15.00, 30),
				new OrderEntity(type2, 64.00, 10),
				new OrderEntity(type2, 15.00, 6),
				new OrderEntity(type3, 80.00, 6),
				new OrderEntity(type3, 64.00, 3)
			);
			orders.forEach(order -> orderService.create(user1.getId(), order));

			log.info("Create default messages values");
			final var messages1 = List.of(
				new MessageEntity("Message1", LocalDateTime.now(), false),
				new MessageEntity("Message2", LocalDateTime.now(), false),
				new MessageEntity("Message3", LocalDateTime.now(), false)
			);
			messages1.forEach(message -> messageService.create(user1.getId(), message));

			final var messages2 = List.of(
				new MessageEntity("Message4", LocalDateTime.now(), false),
				new MessageEntity("Message5", LocalDateTime.now(), false)
			);
			messages2.forEach(message -> messageService.create(user2.getId(), message));

			final var messages3 = List.of(
				new MessageEntity("Message6", LocalDateTime.now(), false),
				new MessageEntity("Message7", LocalDateTime.now(), false)
			);
			messages3.forEach(message -> messageService.create(user3.getId(), message));
		}
	}
}
