package com.example.demo.messages.service;

import java.util.List;
import java.util.stream.StreamSupport;

import org.hibernate.Hibernate;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.PageRequest;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import com.example.demo.core.error.NotFoundException;
import com.example.demo.messages.model.MessageEntity;
import com.example.demo.messages.repository.MessageRepository;
import com.example.demo.users.model.UserEntity;
import com.example.demo.users.service.UserService;

// Бизнес-логика для сущности "Сообщение"
@Service
public class MessageService {
    // Хранилище элементов
    private final MessageRepository repository;

    // Бизнес-логика для отправителей (пользователей)
    private final UserService userService;

    // Конструктор
    public MessageService(MessageRepository repository, UserService userService) {
        this.repository = repository;
        this.userService = userService;
    }

    // Получить все элементы или по заданному фильтру
    @Transactional(readOnly = true)
    public List<MessageEntity> getAll(Long userId) {
        if (userId <= 0L) {
            return StreamSupport.stream(repository.findAll().spliterator(), false).toList();
        }
        else {
            userService.get(userId);
            return repository.findByUserId(userId);
        }
    }

    // Получить все элементы или по заданному фильтру (с пагинацией)
    @Transactional(readOnly = true)
    public Page<MessageEntity> getAll(Long userId, int page, int size) {
        final PageRequest pageRequest = PageRequest.of(page, size);
        if (userId <= 0L) {
            return repository.findAll(pageRequest);
        }
        else {
            userService.get(userId);
            return repository.findByUserId(userId, pageRequest);
        }
    }

    // Получить элемент по идентификатору
    @Transactional(readOnly = true)
    public MessageEntity get(Long userId, Long id) {
        userService.get(userId);
        return repository.findOnyByUserIdAndId(userId, id)
            .orElseThrow(() -> new NotFoundException(MessageEntity.class, id));
    }

    // Получить элемент по идентификатору
    @Transactional(readOnly = true)
    public MessageEntity get(Long id) {
        MessageEntity message = repository.findById(id)
            .orElseThrow(() -> new NotFoundException(MessageEntity.class, id));
        Hibernate.initialize(message.getUser().getMessages());
        return message;
    }

    // Создать элемент
    @Transactional
    public MessageEntity create(Long userId, MessageEntity entity) {
        if (entity == null) {
            throw new IllegalArgumentException("Entity is null");
        }
        final UserEntity existsEntity = userService.get(userId);
        entity.setUser(existsEntity);
        return repository.save(entity);
    }

    // Изменить элемент
    @Transactional
    public MessageEntity update(Long userId, Long id, MessageEntity entity) {
        userService.get(userId);
        final MessageEntity existsEntity = get(userId, id);
        existsEntity.setUser(entity.getUser());
        existsEntity.setText(entity.getText());
        existsEntity.setIsPublished(entity.getIsPublished());
        return repository.save(existsEntity);
    }

    // Удалить элемент
    @Transactional
    public MessageEntity delete(Long userId, Long id) {
        userService.get(userId);
        final MessageEntity existsEntity = get(userId, id);
        repository.delete(existsEntity);
        return existsEntity;
    }

    // Удалить все элементы
    @Transactional
    public void deleteAll() {
        repository.deleteAll();
    }
}
