package com.example.demo.users.api;

import org.modelmapper.ModelMapper;
import org.springframework.security.core.annotation.AuthenticationPrincipal;
import org.springframework.stereotype.Controller;
import org.springframework.ui.Model;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.servlet.mvc.support.RedirectAttributes;

import com.example.demo.core.api.PageAttributesMapper;
import com.example.demo.core.configuration.Constants;
import com.example.demo.core.security.UserPrincipal;
import com.example.demo.messages.api.MessageDto;
import com.example.demo.messages.model.MessageEntity;
import com.example.demo.messages.service.MessageService;
import com.example.demo.orders.api.OrderDto;
import com.example.demo.orders.api.OrderGroupedDto;
import com.example.demo.orders.model.OrderEntity;
import com.example.demo.orders.model.OrderGrouped;
import com.example.demo.orders.service.OrderService;
import com.example.demo.types.api.TypeDto;
import com.example.demo.types.model.TypeEntity;
import com.example.demo.types.service.TypeService;

// Контроллер для профиля пользователя
@Controller
public class UserProfileController {
    // URL для доступа к методам контроллера
    private static final String PROFILE_VIEW = "profile";
 
    // Атрибут модели для пагинации
    private static final String PAGE_ATTRIBUTE = "page";

    // Атрибут модели для обработки данных
    private static final String TYPEID_ATTRIBUTE = "typeId";

    // Бизнес-логика для сущости "Заказ"
    private final OrderService orderService;

    // Бизнес-логика для сущости "Тип" (Тип товара)
    private final TypeService typeService;

    // Бизнес-логика для сущости "Сообщение"
    private final MessageService messageService;

    // Библиотека для преобразования сущности
    private final ModelMapper modelMapper;

    public UserProfileController(
            OrderService orderService,
            TypeService typeService,
            MessageService messageService,
            ModelMapper modelMapper) {
        this.orderService = orderService;
        this.typeService = typeService;
        this.messageService = messageService;
        this.modelMapper = modelMapper;
    }

    private OrderDto toDto(OrderEntity entity) {
        return modelMapper.map(entity, OrderDto.class);
    }

    private OrderGroupedDto toGroupedDto(OrderGrouped entity) {
        return modelMapper.map(entity, OrderGroupedDto.class);
    }

    private TypeDto toTypeDto(TypeEntity entity) {
        return modelMapper.map(entity, TypeDto.class);
    }

    private MessageDto toMessageDto(MessageEntity entity) {
        return modelMapper.map(entity, MessageDto.class);
    }

    @GetMapping
    public String getProfile(
            @RequestParam(name = PAGE_ATTRIBUTE, defaultValue = "0") int page,
            @RequestParam(name = TYPEID_ATTRIBUTE, defaultValue = "0") int typeId,
            Model model,
            @AuthenticationPrincipal UserPrincipal principal) {
        final long userId = principal.getId();
        model.addAttribute(PAGE_ATTRIBUTE, page);
        model.addAttribute(TYPEID_ATTRIBUTE, typeId);

        model.addAllAttributes(PageAttributesMapper.toAttributes(
                "order",
                orderService.getAll(userId, typeId, page, Constants.DEFAULT_PAGE_SIZE),
                this::toDto));

        model.addAttribute("stats",
                orderService.getTotal(userId).stream()
                        .map(this::toGroupedDto)
                        .toList());

        model.addAttribute("types",
                typeService.getAll().stream()
                        .map(this::toTypeDto)
                        .toList());

        model.addAllAttributes(PageAttributesMapper.toAttributes(
                "message",
                messageService.getAll(userId, page, Constants.DEFAULT_PAGE_SIZE),
                this::toMessageDto));
        return PROFILE_VIEW;
    }

    @PostMapping("/delete/{id}")
    public String deleteOrder(
            @PathVariable(name = "id") Long id,
            @RequestParam(name = PAGE_ATTRIBUTE, defaultValue = "0") int page,
            @RequestParam(name = TYPEID_ATTRIBUTE, defaultValue = "0") int typeId,
            RedirectAttributes redirectAttributes,
            @AuthenticationPrincipal UserPrincipal principal) {
        redirectAttributes.addAttribute(PAGE_ATTRIBUTE, page);
        redirectAttributes.addAttribute(TYPEID_ATTRIBUTE, typeId);
        orderService.delete(principal.getId(), id);
        return Constants.REDIRECT_VIEW + "/";
    }
}
