package com.example.demo.users.service;

import java.util.List;
import java.util.Optional;
import java.util.stream.StreamSupport;

import org.springframework.data.domain.Page;
import org.springframework.data.domain.PageRequest;
import org.springframework.security.core.userdetails.UserDetails;
import org.springframework.security.core.userdetails.UserDetailsService;
import org.springframework.security.core.userdetails.UsernameNotFoundException;
import org.springframework.security.crypto.password.PasswordEncoder;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.StringUtils;

import com.example.demo.core.configuration.Constants;
import com.example.demo.core.error.NotFoundException;
import com.example.demo.core.security.UserPrincipal;
import com.example.demo.users.model.UserEntity;
import com.example.demo.users.model.UserRole;
import com.example.demo.users.repository.UserRepository;

// Бизнес-логика для сущности "Пользователь"
@Service
public class UserService implements UserDetailsService {
    // Хранилище элементов
    private final UserRepository repository;

    // Шифрование паролей
    private final PasswordEncoder passwordEncoder;

    // Конструктор
    public UserService(
            UserRepository repository,
            PasswordEncoder passwordEncoder) {
        this.repository = repository;
        this.passwordEncoder = passwordEncoder;
    }

    // Проверка уникальности имени/логина пользователя
    private void checkUsername(Long id, String username) {
        final Optional<UserEntity> existsUser = repository.findByUsername(username);
        if (existsUser.isPresent() && !existsUser.get().getId().equals(id)) {
            throw new IllegalArgumentException(
                    String.format("User with username %s is already exists", username));
        }
    }

    // Проверка уникальности адреса электронной почты пользователя
    private void checkEmail(Long id, String email) {
        final Optional<UserEntity> existsUser = repository.findByEmail(email);
        if (existsUser.isPresent() && !existsUser.get().getId().equals(id)) {
            throw new IllegalArgumentException(
                    String.format("User with email %s is already exists", email));
        }
    }

    // Получить все элементы
    @Transactional(readOnly = true)
    public List<UserEntity> getAll() {
        return StreamSupport.stream(repository.findAll().spliterator(), false).toList();
    }

    // Получить все элементы (с пагинацией)
    @Transactional(readOnly = true)
    public Page<UserEntity> getAll(int page, int size) {
        final PageRequest pageRequest = PageRequest.of(page, size);
        return repository.findAll(pageRequest);
    }

    // Получить элемент по идентификатору
    @Transactional(readOnly = true)
    public UserEntity get(Long id) {
        return repository.findById(id)
                .orElseThrow(() -> new NotFoundException(UserEntity.class, id));
    }

    // Получить элемент по имени/логину
    @Transactional(readOnly = true)
    public UserEntity getByUsername(String username) {
        return repository.findByUsername(username)
                .orElseThrow(() -> new IllegalArgumentException("Invalid username"));
    }

    // Создать элемент
    @Transactional
    public UserEntity create(UserEntity entity) {
        if (entity == null) {
            throw new IllegalArgumentException("Entity is null");
        }

        checkUsername(null, entity.getUsername());
        checkEmail(null, entity.getEmail());

        final String password = Optional.ofNullable(entity.getPassword()).orElse("");
        entity.setPassword(
                passwordEncoder.encode(
                        StringUtils.hasText(password.strip()) ? password : Constants.DEFAULT_PASSWORD));
        entity.setRole(Optional.ofNullable(entity.getRole()).orElse(UserRole.USER));

        return repository.save(entity);
    }

    // Изменить элемент
    @Transactional
    public UserEntity update(Long id, UserEntity entity) {
        final UserEntity existsEntity = get(id);
        existsEntity.setUsername(entity.getUsername());
        existsEntity.setEmail(entity.getEmail());
        return repository.save(existsEntity);
    }

    // Удалить элемент
    @Transactional
    public UserEntity delete(Long id) {
        final UserEntity existsEntity = get(id);
        repository.delete(existsEntity);
        return existsEntity;
    }

    // Удалить все элементы
    @Transactional
    public void deleteAll() {
        repository.deleteAll();
    }

    // Загрузить пользователя по имени/логину
    @Override
    @Transactional(readOnly = true)
    public UserDetails loadUserByUsername(String username) throws UsernameNotFoundException {
        final UserEntity existsUser = getByUsername(username);
        return new UserPrincipal(existsUser);
    }
}
